package com.shigongli.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.shigongli.annotations.ParamNotEmpty;
import com.shigongli.common.ResponseBean;
import com.shigongli.common.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.Parameter;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@Component
public class ParamNotEmptyInterceptor extends HandlerInterceptorAdapter {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }

        HandlerMethod hm = (HandlerMethod) handler;

        // 获取控制器方法参数列表
        Parameter[] parameters = hm.getMethod().getParameters();
        if (parameters.length < 1) {
            return true;
        }

        // 遍历所有参数
        for (Parameter parameter: parameters) {
            if (null == parameter) {
                continue;
            }

            // 遍历所有参数字段
            Field[] declaredFields = parameter.getClass().getDeclaredFields();
            if (declaredFields.length < 1) {
                continue;
            }

            for (Field field : declaredFields) {
                // 查找带有 ParamNotNull 注解的字段
                ParamNotEmpty annotation = field.getAnnotation(ParamNotEmpty.class);
                if (null == annotation) {
                    continue;
                }

                boolean isEmpty = false;
                Object val = field.get(parameter);
                if (null == val) {
                    isEmpty = true;
                } else if (field.getType().equals(String.class)) {
                    // 字符串
                    isEmpty = StringUtils.isEmpty(val.toString());
                } else if (null != field.getType().getComponentType()) {
                    // 数组
                    isEmpty = Array.getLength(val) < 1;
                } else if (isCollection(field.getType())) {
                    // 集合
                    int size = (int) field.getType().getDeclaredMethod("size", int.class).invoke(val);
                    isEmpty = size < 1;
                } else {
                    // others
                }

                if (isEmpty) {
                    String errMessage = annotation.name() + " 不能为空";
                    response.addHeader("Content-type", "application/json");
                    response.getOutputStream().write(
                            JSONObject.toJSONBytes(
                                    ResponseBean.error(errMessage, ResponseBean.ERROR_MISSING_PARAMS)
                            ));
                    return false;
                }
            }
        }

        return true;
    }

    private boolean isCollection(Class cla) {
        return cla.equals(List.class)
                || cla.equals(Map.class)
                || cla.equals(Set.class);
    }
}
